@@ -34,7 +34,6 @@
 #include <snapper/SnapperTmpl.h>
 #include <snapper/Enum.h>
 #include <snapper/AsciiFile.h>
-#include <snapper/SystemCmd.h>
 #include <snapper/SnapperDefines.h>
 #include <snapper/XAttributes.h>
 #ifdef ENABLE_ROLLBACK
@@ -92,7 +91,7 @@ bool no_dbus = false;
 
 struct MyFiles : public Files
 {
-    friend class MyComparison;
+    friend struct MyComparison;
 
     MyFiles(const FilePaths* file_paths)
 	: Files(file_paths) {}
@@ -141,6 +140,32 @@ help_list_configs()
 }
 
 
+list<pair<string, string>>
+enum_configs(DBus::Connection* conn)
+{
+    list<pair<string, string>> configs;
+
+    if (no_dbus)
+    {
+	list<ConfigInfo> config_infos = Snapper::getConfigs();
+	for (list<ConfigInfo>::const_iterator it = config_infos.begin(); it != config_infos.end(); ++it)
+	{
+	    configs.push_back(make_pair(it->getConfigName(), it->getSubvolume()));
+	}
+    }
+    else
+    {
+	list<XConfigInfo> config_infos = command_list_xconfigs(*conn);
+	for (list<XConfigInfo>::const_iterator it = config_infos.begin(); it != config_infos.end(); ++it)
+	{
+	    configs.push_back(make_pair(it->config_name, it->subvolume));
+	}
+    }
+
+    return configs;
+}
+
+
 void
 command_list_configs(DBus::Connection* conn, Snapper* snapper)
 {
@@ -158,27 +183,14 @@ command_list_configs(DBus::Connection* conn, Snapper* snapper)
     header.add(_("Subvolume"));
     table.setHeader(header);
 
-    if (no_dbus)
+    list<pair<string, string> > configs = enum_configs(conn);
+
+    for (list<pair<string,string> >::iterator it = configs.begin(); it != configs.end(); ++it)
     {
-	list<ConfigInfo> config_infos = Snapper::getConfigs();
-	for (list<ConfigInfo>::const_iterator it = config_infos.begin(); it != config_infos.end(); ++it)
-	{
-	    TableRow row;
-	    row.add(it->getConfigName());
-	    row.add(it->getSubvolume());
-	    table.add(row);
-	}
-    }
-    else
-    {
-	list<XConfigInfo> config_infos = command_list_xconfigs(*conn);
-	for (list<XConfigInfo>::const_iterator it = config_infos.begin(); it != config_infos.end(); ++it)
-	{
-	    TableRow row;
-	    row.add(it->config_name);
-	    row.add(it->subvolume);
-	    table.add(row);
-	}
+	TableRow row;
+	row.add(it->first);
+	row.add(it->second);
+	table.add(row);
     }
 
     cout << table;
@@ -382,15 +394,20 @@ help_list()
 	 << endl
 	 << _("    Options for 'list' command:") << endl
 	 << _("\t--type, -t <type>\t\tType of snapshots to list.") << endl
+	 << _("\t--all-configs, -a\t\tList snapshots from all accessible configs.") << endl
 	 << endl;
 }
 
+enum ListMode { LM_ALL, LM_SINGLE, LM_PRE_POST };
+
+void list_from_one_config(DBus::Connection* conn, Snapper* snapper, string config_name, ListMode list_mode);
 
 void
 command_list(DBus::Connection* conn, Snapper* snapper)
 {
     const struct option options[] = {
 	{ "type",		required_argument,	0,	't' },
+	{ "all-configs",	no_argument,		0,	'a' },
 	{ 0, 0, 0, 0 }
     };
 
@@ -401,7 +418,6 @@ command_list(DBus::Connection* conn, Snapper* snapper)
 	exit(EXIT_FAILURE);
     }
 
-    enum ListMode { LM_ALL, LM_SINGLE, LM_PRE_POST };
     ListMode list_mode = LM_ALL;
 
     GetOpts::parsed_opts::const_iterator opt;
@@ -421,6 +437,29 @@ command_list(DBus::Connection* conn, Snapper* snapper)
 	}
     }
 
+    list<pair<string, string> > configs;
+    if ((opt = opts.find("all-configs")) != opts.end())
+    {
+	configs = enum_configs(conn);
+    }
+    else
+    {
+	configs.push_back(make_pair(config_name, ""));
+    }
+
+    for (list<pair<string,string> >::iterator it = configs.begin(); it != configs.end(); ++it)
+    {
+	if (it != configs.begin())
+	    cout << endl;
+
+	if (configs.size() > 1)
+	    cout << "Config: " << it->first << ", subvolume: " << it->second << endl;
+	list_from_one_config(conn, snapper, it->first, list_mode);
+    }
+}
+
+void list_from_one_config(DBus::Connection* conn, Snapper* snapper, string config_name, ListMode list_mode)
+{
     Table table;
 
     switch (list_mode)
@@ -928,6 +967,13 @@ command_status(DBus::Connection* conn, Snapper* snapper)
     if (getopts.numArgs() != 1)
     {
 	cerr << _("Command 'status' needs one argument.") << endl;
+
+	if (getopts.numArgs() == 2)
+	{
+	    cerr << _("Maybe you forgot the delimiter '..' between the snapshot numbers.") << endl
+		 << _("See 'man snapper' for further instructions.") << endl;
+	}
+
 	exit(EXIT_FAILURE);
     }
 
@@ -964,6 +1010,10 @@ help_diff()
 {
     cout << _("  Comparing snapshots:") << endl
 	 << _("\tsnapper diff <number1>..<number2> [files]") << endl
+	 << endl
+	 << _("    Options for 'diff' command:") << endl
+	 << _("\t--diff-cmd <command>\t\tCommand used for comparing files.") << endl
+	 << _("\t--extensions, -x <options>\tExtra options passed to the diff command.") << endl
 	 << endl;
 }
 
@@ -971,15 +1021,30 @@ help_diff()
 void
 command_diff(DBus::Connection* conn, Snapper* snapper)
 {
-    GetOpts::parsed_opts opts = getopts.parse("diff", GetOpts::no_options);
+    const struct option options[] = {
+	{ "diff-cmd",		required_argument,	0,	0 },
+	{ "extensions",		required_argument,	0,	'x' },
+	{ 0, 0, 0, 0 }
+    };
 
-    if (getopts.numArgs() < 1) {
+    GetOpts::parsed_opts opts = getopts.parse("diff", options);
+
+    if (getopts.numArgs() < 1)
+    {
 	cerr << _("Command 'diff' needs at least one argument.") << endl;
 	exit(EXIT_FAILURE);
     }
 
+    Differ differ;
+
     GetOpts::parsed_opts::const_iterator opt;
 
+    if ((opt = opts.find("diff-cmd")) != opts.end())
+	differ.command = opt->second;
+
+    if ((opt = opts.find("extensions")) != opts.end())
+	differ.extensions = opt->second;
+
     pair<unsigned int, unsigned int> nums(read_nums(getopts.popArg()));
 
     MyComparison comparison(*conn, nums, true);
@@ -988,14 +1053,7 @@ command_diff(DBus::Connection* conn, Snapper* snapper)
     if (getopts.numArgs() == 0)
     {
 	for (Files::const_iterator it1 = files.begin(); it1 != files.end(); ++it1)
-	{
-	    SystemCmd cmd(DIFFBIN " --unified --new-file " + quote(it1->getAbsolutePath(LOC_PRE)) +
-			  " " + quote(it1->getAbsolutePath(LOC_POST)), false);
-
-	    const vector<string> lines = cmd.stdout();
-	    for (vector<string>::const_iterator it2 = lines.begin(); it2 != lines.end(); ++it2)
-		cout << it2->c_str() << endl;
-	}
+	    differ.run(it1->getAbsolutePath(LOC_PRE), it1->getAbsolutePath(LOC_POST));
     }
     else
     {
@@ -1004,15 +1062,8 @@ command_diff(DBus::Connection* conn, Snapper* snapper)
 	    string name = getopts.popArg();
 
 	    Files::const_iterator it1 = files.findAbsolutePath(name);
-            if (it1 == files.end())
-                continue;
-
-	    SystemCmd cmd(DIFFBIN " --unified --new-file " + quote(it1->getAbsolutePath(LOC_PRE)) +
-			  " " + quote(it1->getAbsolutePath(LOC_POST)), false);
-
-	    const vector<string> lines = cmd.stdout();
-	    for (vector<string>::const_iterator it2 = lines.begin(); it2 != lines.end(); ++it2)
-		cout << it2->c_str() << endl;
+	    if (it1 != files.end())
+		differ.run(it1->getAbsolutePath(LOC_PRE), it1->getAbsolutePath(LOC_POST));
 	}
     }
 }
@@ -1412,7 +1463,8 @@ command_xa_diff(DBus::Connection* conn, Snapper* snapper)
 {
     GetOpts::parsed_opts opts = getopts.parse("xadiff", GetOpts::no_options);
 
-    if (getopts.numArgs() < 1) {
+    if (getopts.numArgs() < 1)
+    {
         cerr << _("Command 'xadiff' needs at least one argument.") << endl;
         exit(EXIT_FAILURE);
     }
@@ -1423,9 +1475,11 @@ command_xa_diff(DBus::Connection* conn, Snapper* snapper)
     MyFiles& files = comparison.files;
 
     if (getopts.numArgs() == 0)
-        for (Files::const_iterator it1 = files.begin(); it1 != files.end(); ++it1)
+    {
+	for (Files::const_iterator it1 = files.begin(); it1 != files.end(); ++it1)
             if (it1->getPreToPostStatus() & XATTRS)
                 print_xa_diff(it1->getAbsolutePath(LOC_PRE), it1->getAbsolutePath(LOC_POST));
+    }
     else
     {
         while (getopts.numArgs() > 0)
@@ -1691,6 +1745,13 @@ main(int argc, char** argv)
 	}
 	catch (const DBus::ErrorException& e)
 	{
+	    if (strcmp(e.name(), "error.unknown_config") == 0 && config_name == "root")
+	    {
+		cerr << _("The config 'root' does not exist. Likely snapper is not configured.") << endl
+		     << _("See 'man snapper' for further instructions.") << endl;
+		exit(EXIT_FAILURE);
+	    }
+
 	    cerr << error_description(e) << endl;
 	    exit(EXIT_FAILURE);
 	}
