@@ -545,13 +545,6 @@ Client::signal_snapshots_deleted(DBus::Connection& conn, const string& config_na
 }
 
 
-struct UnknownFile : public std::exception
-{
-    explicit UnknownFile() throw() {}
-    virtual const char* what() const throw() { return "unknown config"; }
-};
-
-
 void
 Client::list_configs(DBus::Connection& conn, DBus::Message& msg)
 {
@@ -1504,11 +1497,6 @@ Client::dispatch(DBus::Connection& conn, DBus::Message& msg)
 	DBus::MessageError reply(msg, "error.delete_snapshot_failed", DBUS_ERROR_FAILED);
 	conn.send(reply);
     }
-    catch (const UnknownFile& e)
-    {
-	DBus::MessageError reply(msg, "error.unknown_file", DBUS_ERROR_FAILED);
-	conn.send(reply);
-    }
     catch (const InvalidConfigdataException& e)
     {
 	DBus::MessageError reply(msg, "error.invalid_configdata", DBUS_ERROR_FAILED);
@@ -1617,11 +1605,7 @@ Clients::add(const string& name)
 {
     assert(find(name) == entries.end());
 
-#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 4)
     entries.emplace_back(name);
-#else
-    entries.push_back(name);
-#endif
 
     return --entries.end();
 }
